package core;

import java.io.IOException;
import java.io.InputStreamReader;
import java.io.Serializable;
import java.net.ServerSocket;
import java.net.Socket;

/*
 * Created on May 18, 2003
 */

/**
 * This class represents an agent running a protocol
 * @author Xavier Didelot
 */
public class Agent implements Serializable {



	/**
	 * The identity of the agent as written in the protocol description (for example a or b)
	 */
	private String id;
	/**
	 * The internet location of the agent (for example booth35.ecs.ox.ac.uk)
	 */
	private String ip;
	/**
	 * The name of the agent that is used for example in the KeyStores (for example Alice or Bob)
	 */
	private String name;
	/**
	 * This PrintWriter is used by an host to send a message to another one, with the put method
	 */
	private PrintWriterS out = null;
	/**
	 * This BufferedReader is used by an host to receive messages that have been sent to him, with the get method
	 */
	private BufferedReaderS in = null;
	/**
	 * The socket used to communicate with this Agent
	 */
	private Socket socket = null;

	/**
	 * Creates a new empty agent
	 */
	public Agent() {
		super();
		id = "";
		name = "";
	}

	/**
	 * Changes the IP of this Agent
	 * @param ip new IP to be allocated to this host
	 */
	public void setIP(String ip) {
		this.ip = ip;
	}

	/**
	 * Returns the id of this agent
	 * @return the id of this agent
	 */
	public String getId() {
		return id;
	}

	/**
	 * return the name of this agent
	 * @return the name of this agent
	 */
	public String getName() {
		return name;
	}

	/**
	 * sets the id of this agent
	 * @param string new id of this agent
	 */
	public void setId(String string) {
		id = string;
	}

	/**
	 * sets the name of this agent
	 * @param string new name of this agent
	 */
	public void setName(String string) {
		name = string;
	}

	/**
	 * sets the in and out private members according to the socket used
	 */
	private void setInAndOut() {
		try {
			out = new PrintWriterS(socket.getOutputStream(), true);
			in =
				new BufferedReaderS(
					new InputStreamReader(socket.getInputStream()));
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	/**
	 * sets the socket used to communicate with this Agent and the in and out accordingly
	 * @param socket to use to communicate with this Agent
	 */
	public void setSocket(Socket socket) {
		this.socket = socket;
		if (socket!=null) setInAndOut();
	}

	/**
	 * creates a socket connected to (url,port) and use it to communicate with this Agent
	 * also sets the in and out accordingly
	 * @param url internet name of the host corresponding to this Agent
	 * @param port port to use for communication with this Agent
	 */
	public void contact(String url, int port) {
		try {
			socket = new Socket(url, port);
		} catch (Exception e) {
			e.printStackTrace();
		}
		setInAndOut();
	}

	/**
	 * Send data to this Agent on the given port. A socket or an IP has to be allocated to this Agent.
	 * @param what what has to be sent to the Agent
	 * @param port port to use
	 */
	public void put(String what,int port) {
		if (out == null) {
			try {
				setSocket(new Socket(ip, port));
			} catch (Exception e1) {
				e1.printStackTrace();
			}
		}
		out.println(what);
	}

	/**
	 * Receive data from this Agent. A socket has to be allocated to this Agent.
	 * @return the String received from this Agent
	 */
	public String get(int port) {
		if (socket == null) {
			try {
				//System.out.println("waiting for a connection to be established");
				ServerSocket ss = new ServerSocket(port);
				Socket clientSocket = null;
				clientSocket = ss.accept();
				setSocket(clientSocket);
				ss.close();
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
		String result = null;
		try {
			result = in.readLine();
			while (in.ready())
				result = result + in.readLine();
		} catch (IOException e) {
			e.printStackTrace();
		}
		return result;
	}

	/**
	 * Return the IP of this Agent
	 * @return the IP of this Agent
	 */
	public String getIp() {
		return ip;
	}

	/**
	 * Return the Socket that has been allocated to this Agent
	 * @return the Socket that has been allocated to this Agent
	 */
	public Socket getSocket() {
		return socket;
	}

}
