package core;

import java.io.IOException;
import java.lang.reflect.Array;
import java.security.Key;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.security.interfaces.RSAKey;

import javax.crypto.Cipher;

import sun.misc.BASE64Decoder;
import sun.misc.BASE64Encoder;

/**
 * This class represents a Message
 * @author Xavier Didelot
 */
public class Message implements IMessage {

	/**
	 * This string represents the content of the message
	 */
	private String string;

	/**
	 * Creates a new empty message
	 */
	public Message() {
		super();
		string = "";
	}

	/**
	 * Creates a new message containing the given string
	 * It is advised to only use Base64 string in the message to avoid any issue 
	 * @param string contained in the message to create
	 */
	/*public Message(String string) {
		super();
		this.string = string;
	}*/
	
	public Message(Agent agent) {
		string=agent.getName();
	}
	
	public Message(INonce nonce) {
		string=nonce.getString();
	}

	/**
	 * Creates a new message containing the two given Messages contatenated
	 * @param m1 first message
	 * @param m2 second message
	 */
	public Message(Message m1, Message m2) {
		super();
		this.string = m1.getString();
		addMessage(m2);
	}

	/**
	 * This method returns the value of the protected member string
	 * It is especially useful to send a message to another host
	 * @return the content of the message
	 */
	public String getString() {
		return string;
	}

	/**
	 * Change the value of the private member string into the given string
	 * It is advised to only use Base64 string in the message to avoid any issue 
	 * @param string new value for the content of the message
	 */
	public void setString(String string) {
		this.string = string;
	}

	/**
	 * This method concats two messages and put an 'at' symbol between them so that we can
	 * divide it later with the method getAllFields for example
	 * This means that a Message should never contain the symbol 'at'
	 * It is advised to only use Base64 string in the message to avoid any issue 
	 * @param msg
	 */
	public void addMessage(Message msg) {
		if (this.string.compareTo("") == 0)
			this.string = msg.getString();
		else
			this.string = this.string + "@" + msg.getString();
	}

	/**This method tells how many submessages a message contain
	 * This method should not be used directly, cf getAllFields
	 * @return the number of messages that have been concatenated in order to create this message
	 */
	private int getNbFields() {
		int result = 1;
		String str = string;
		while (str.indexOf('@') != -1) {
			result++;
			str = str.substring(str.indexOf('@') + 1);
		}
		return result;
	}

	/**
	 * Returns a given part of the message
	 * This method should not be used directly, cf getAllFields instead
	 * @param field number of the field to get
	 * @return the selected part of a message
	 * @throws Exception if the message does not contain the part that has been asked for
	 */
	private Message getField(int field) throws Exception {
		String str = string;
		int whichField = field;
		while (whichField > 1) {
			whichField--;
			if (str.indexOf('@') == -1)
				throw new Exception("No such field");
			str = str.substring(str.indexOf('@') + 1);
		}
		if (str.indexOf('@') == -1)
			{Message m=new Message();m.setString(str);return m;} 
		else
			{Message m=new Message();m.setString(str.substring(0, str.indexOf('@')));return m;} 
	}

	/**
	 * Returns an array containing the different parts of a message, ie the different
	 * messages that have been concatenated in order to build this message
	 * @return an array containing the different parts of a message
	 */
	public IMessage[] getAllFields() {
		int nb = getNbFields();
		Message [] result=null;
		try {
		result = (Message[]) Array.newInstance(Class.forName("core.Message"), nb);
		int i;
			for (i = 0; i < nb; i++)
				result[i] = getField(i + 1);
		} catch (Exception e) {
			e.printStackTrace();
		}
		return result;
	}

	/**
	 * Encrypts the message with the given key, using the given algorithm, and put the result in Base64 format
	 * @param algorithm algorithm to use for encryption
	 * @param key key to use for encryption
	 * @throws Exception
	 */
	public void encrypt(String algorithm, Key key)
		throws
			Exception {
		
		byte[] in=string.getBytes("UTF8");
		byte[] out;
		if (algorithm.compareTo("RSA")==0) {
			out=RSALib.RSAenc(in,(RSAKey)key);
		} else {
		Cipher cipher = Cipher.getInstance(algorithm);
		cipher.init(Cipher.ENCRYPT_MODE, key);
			out = cipher.doFinal(in);
		}
		
		BASE64Encoder encoder = new BASE64Encoder();
		string = encoder.encode(out);
	}

	/**
	 * Decrypt the message with a given key, using the given algorithm. The message has to be in Base64 format.
	 * @param algorithm algorithm to use for decryption
	 * @param key key to use for decryption
	 * @throws Exception
	 */
	public void decrypt(String algorithm, Key key)
		throws
			Exception
			 {
				BASE64Decoder decoder = new BASE64Decoder();
				byte[] in = decoder.decodeBuffer(string);
				byte[] out;
				if (algorithm.compareTo("RSA")==0) {
					out=RSALib.RSAdec(in,(RSAKey)key);
				} else {
				Cipher cipher = Cipher.getInstance(algorithm);
				cipher.init(Cipher.DECRYPT_MODE, key);
					out = cipher.doFinal(in);
				}
		
				string = new String(out, "UTF8");
			}			 	

	/**
	 * This method hashes the Message
	 * @throws NoSuchAlgorithmException
	 */
	public void hash(String method) throws NoSuchAlgorithmException {
		MessageDigest md = MessageDigest.getInstance(method);
		md.update(getString().getBytes());
		BASE64Encoder encoder = new BASE64Encoder();
		String str = encoder.encode(md.digest());
		setString(str);
	}

	/**
	 * Creates a copy of the Message
	 * @see java.lang.Object#clone()
	 */
	protected Object clone() throws CloneNotSupportedException {
		Message m=new Message();
		m.setString(this.getString());
		return m;
	}

	/**
	 * Check if the Message is equal to another one
	 * @param other the other Message
	 * @return true if the messages are the same, false otherwise
	 */
	public boolean equals(Message other) {
		return string.compareTo(other.getString()) == 0;
	}

	public void xor() throws Exception {
		IMessage[] fields = getAllFields();
		int length=fields.length;
		if (length<2) throw new Exception("Impossible to XOR");
		fields[length-2].xor(fields[length-1]);
		IMessage m=fields[0];
		for (int i=1;i<length-1;i++) m.addMessage((Message)fields[i]);
		setString(m.getString());
	}

	public void xor(IMessage message) {
		byte[] ba=getString().getBytes();
		ba=Message.xor(ba,message.getString().getBytes());
		BASE64Encoder encoder = new BASE64Encoder();
		String str = encoder.encode(ba);
		setString(str);
	}
	
	public void unxor(IMessage message) {
		BASE64Decoder decoder = new BASE64Decoder();
		byte[] ba=null;
		try {
			ba = decoder.decodeBuffer(getString());
		} catch (IOException e) {e.printStackTrace();}	
		ba=Message.xor(ba,message.getString().getBytes());	
		setString(new String(ba));
	}

	protected static byte[] xor(byte[] ba, byte[] bs) {
		byte[] b1;
		byte[] b2;
		if (ba.length>bs.length) {b1=ba;b2=bs;} else {b1=bs;b2=ba;}
		for (int i=0;i<b2.length;i++) {
			b1[i]=(byte)(b1[i] ^b2[i]);
		}
		return b1;
	}

	public void setStringAddr(String string) {
		setString(string);		
	}

	public void encrypt(String algorithm, Key key, String protoTag) throws Exception {
		setString(getString()+"/"+protoTag);
		encrypt(algorithm,key);
	}

	public void decrypt(String algorithm, Key key, String protoTag) throws Exception {
		decrypt(algorithm,key);
		String str=getString();
		int at=str.lastIndexOf('/');
		if (protoTag.compareTo(str.substring(at+1))!=0) 
			throw new Exception("Incorrect protocol tag "+protoTag+" "+str.substring(at+1));
		else System.out.println("Protocol tag correct");
		setString(str.substring(0,at)); 
	}


	public void hash(String method, String protoTag) throws NoSuchAlgorithmException {
		setString(getString()+"/"+protoTag);
		hash(method);		
	}
	
	
	
}		