package core;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.security.Key;
import java.security.NoSuchAlgorithmException;
import java.security.SecureRandom;

import javax.crypto.KeyGenerator;

import sun.misc.BASE64Decoder;
import sun.misc.BASE64Encoder;

/*
 * Created on May 30, 2003
 */

/**
 * This class represents a Message which is a Key. It is useful for protocols in which a Key is sent from one Agent to another.
 * @author Xavier Didelot
 */
public class MessageKey extends Message implements IMessageKey {

	/**
	 * Create an empty MessageKey
	 */
	public MessageKey() {
	}

	/**
	 * Create a new MessageKey containing a fresh key generated with the given algorithm
	 * @param algo algorithm to use for generation of the fresh key
	 * @throws IOException
	 * @throws NoSuchAlgorithmException
	 */
	public MessageKey(String algo)
		throws IOException, NoSuchAlgorithmException {
		super();
		KeyGenerator generator = null;
		generator = KeyGenerator.getInstance(algo);
		generator.init(new SecureRandom());
		Key key = generator.generateKey();
		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		ObjectOutputStream s = new ObjectOutputStream(baos);
		s.writeObject(key);
		byte[] array = baos.toByteArray();
		s.close();
		BASE64Encoder encoder = new BASE64Encoder();
		String result = encoder.encode(array);
		setString(result);
	}

	/**
	 * Create a MessageKey containing a Key
	 * @param key Key that has to be contained in the Message
	 * @throws IOException
	 */
	public MessageKey(Key key) throws IOException {
		super();
		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		ObjectOutputStream s = new ObjectOutputStream(baos);
		s.writeObject(key);
		byte[] array = baos.toByteArray();
		s.close();
		BASE64Encoder encoder = new BASE64Encoder();
		String result = encoder.encode(array);
		setString(result);
	}

	public MessageKey(Message msg) {
		setString(msg.getString());
	}
	
	/**
	 * Convert this MessageKey into a Key
	 * @return the Key contained in this Message
	 * @throws IOException
	 * @throws ClassNotFoundException
	 */
	public Key toKey() throws IOException, ClassNotFoundException {
		BASE64Decoder decoder = new BASE64Decoder();
		byte[] raw = decoder.decodeBuffer(getString());
		ByteArrayInputStream bais = new ByteArrayInputStream(raw);
		ObjectInputStream s = new ObjectInputStream(bais);
		Key key = (Key) s.readObject();
		return key;
	}

	/**
	 * Reads a key from a file
	 * @param file file where is stored the key
	 * @return Key stored in the file
	 * @throws IOException
	 * @throws ClassNotFoundException
	 */
	static public Key keyFromFile(String file)
		throws IOException, ClassNotFoundException {
		ObjectInputStream s = new ObjectInputStream(new FileInputStream(file));
		Key key = (Key) s.readObject();
		s.close();
		return key;
	}

	/**
	 * Store a Key into a File
	 * @param key Key to store in the File
	 * @param file file where the Key has to be stored
	 * @throws IOException
	 */
	static public void storeKeyToFile(Key key, String file)
		throws IOException {
		ObjectOutputStream s =
			new ObjectOutputStream(new FileOutputStream(file));
		s.writeObject(key);
		s.close();
	}

	/**
	 * This static method returns a fresh Key created with the given algorithm
	 * @param algo algoritm to use for creation of a fresh key
	 * @return a fresh key
	 * @throws NoSuchAlgorithmException
	 */
	static public Key createKey(String algo) throws NoSuchAlgorithmException {
		KeyGenerator generator = KeyGenerator.getInstance(algo);
		generator.init(new SecureRandom());
		Key key = generator.generateKey();
		return key;
	}

	/**
	 * This sample method shows how to store a Key in a file 
	 * @param args the Array args must contain two fields: firstly the algorithm to use to create a fresh Key and secondly the name of the file where to store the generated key
	 */
	public static void main(String[] args) {
		if (args.length != 2) {
			System.out.println("usage: java MessageKey algo file");
			System.exit(0);
		}
		Key key;
		try {
			key = createKey(args[0]);
			storeKeyToFile(key, args[1]);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

}
